/*********************************************************************
 * S_Socket.H -- simple AF_INET socket class
 * $Id: S_Llist.H,v 1.2 1998/04/10 17:56:29 tjko Exp $
 *
 * Copyright (c) 1998  Timo Kokkonen <tjko@iki.fi>
 *
 *
 * This file may be copied under the terms and conditions 
 * of the GNU General Public License, as published by the Free
 * Software Foundation (Cambridge, Massachusetts).
 */

#ifndef _S_LLIST_H_
#define _S_LLIST_H_

#include <stdio.h>

class S_Item {

public:
  friend class S_Llist;

  S_Item() { next_=prev_=0; };
  virtual ~S_Item() {};
  virtual S_Item *duplicate() const = 0;
  S_Item *next() { return next_; };
  S_Item *prev() { return prev_; };

private:
  S_Item *next_;
  S_Item *prev_;
};


class S_Llist {

public:
  ~S_Llist();
  S_Llist(int type = 0); // if type=1 list manages items

  S_Item *begin() { return list; };
  S_Item *end() { return last; };
  S_Item *item(unsigned long index);

  void push(S_Item *i, S_Item *place) { push_(i,place,0); };
  void push_end(S_Item *i) { push_(i,end(),0); };
  void push_begin(S_Item *i) { push_(i,begin(),1); };
  void pop(S_Item *i);
  void pop(unsigned long index) { pop(item(index)); };
  S_Item *operator[] (unsigned long index) { return item(index); };

  void add(S_Item *i) { push_end(i); };
  void del(S_Item *i) { pop(i); };
  void del(unsigned long index) { pop(index); };

  unsigned long size() { return items; };
  

private:
  void push_(S_Item *i, S_Item *place, int first);

  S_Item          *list;
  S_Item          *last; 
  int              mode;    
  unsigned long    items;
};



#endif
