/*********************************************************************
 * jyudesk -- main program
 * $Id: configuration.C,v 1.12 1999/02/02 15:30:15 pjka Exp $
 *
 * Copyright (c) 1998  Timo Kokkonen <tjko@iki.fi>
 *
 *
 * This file may be copied under the terms and conditions 
 * of the GNU General Public License, as published by the Free
 * Software Foundation (Cambridge, Massachusetts).
 */

#include <stdlib.h>
#include <stdio.h>
#include <string.h>
#include <sys/types.h>
#include <time.h>

#include "main.H"

static char *rcsid = "$Id: configuration.C,v 1.12 1999/02/02 15:30:15 pjka Exp $";

// temperature.C
extern char *t_addr;
extern char *t_port;
extern char *t_url;
extern char *t_search1;
extern char *t_search2;
extern char *t_search3;
extern char *t_label2;
extern char *t_label3;
extern int  t_timeout;
static char *t_url2 = NULL;

// phonebook.C
extern char *pb_addr;
extern char *pb_port;

// pop3.C
extern char *pop3_host;
extern char *pop3_user;
extern int pop3_enabled;
extern char *pop3_pwd;

// main.C
extern char *xlock_cmd;
extern char *keymap1_cmd;
extern char *keymap2_cmd;
extern char *keymap1_name;
extern char *keymap2_name;
extern char *keymap_label;
extern int  draw_border;
extern int  clock1_dif;
extern int  clock2_dif;
extern char *clock1_tz;
extern char *clock2_tz;
extern int  RGB_mode;

extern char *conf_file;



struct confinfo_t {
  char *tag;
  char **var;
  int  *var_int;
};

confinfo_t confinfo[] = {
  {"border", 0, &draw_border},
  {"pb_addr", &pb_addr,0},
  {"pb_port", &pb_port,0},
  {"t_url", &t_url2,0},
  {"t_search1", &t_search1,0},
  {"t_search2", &t_search2,0},
  {"t_search3", &t_search3,0},
  {"t_timeout", 0,&t_timeout},
  {"t_label2", &t_label2, 0},
  {"t_label3", &t_label3, 0},
  {"xlock", &xlock_cmd,0},
  {"keymap1", &keymap1_cmd,0},
  {"keymap2", &keymap2_cmd,0},
  {"keymap1_name", &keymap1_name,0},
  {"keymap2_name", &keymap2_name,0},
  {"keymap_label", &keymap_label,0},
  {"clock1_tz", &clock1_tz,0},
  {"clock1_dif", 0, &clock1_dif},
  {"clock2_tz", &clock2_tz,0},
  {"clock2_dif", 0, &clock2_dif},
  {"RGB_mode", 0, &RGB_mode},
  {"POP3_enabled", 0, &pop3_enabled},
  {"POP3_host", &pop3_host, 0},
  {"POP3_user", &pop3_user, 0},

  {NULL,NULL}
};


int split_url(const char *url_o, char **host, char **service, char **location)
{
  char *s,*s2,*url;

  if (!url_o) return -2;
  
  url = strdup(url_o);
  s = strstr(url,"://");
  if (!s) return -1;

  *s=0; s+=3;
  *service=strdup(url);

  s2 = strstr(s,"/");
  if (!s2) 
    *location=strdup("/");
  else {
    *location=strdup(s2);
    *s2=0;
  }

  *host=strdup(s);

  free(url);
  return 0;
}


void set_defaults() {
  if (rcsid);
  pb_addr = strdup("phone.jyu.fi");
  pb_port = strdup("1212");
  t_addr = strdup("weather.jyu.fi");
  t_port = strdup("http");
  t_url  = strdup("/");
  t_url2 = strdup("http://weather.jyu.fi/");
  t_search1 = strdup("ulko):");
  t_search2 = strdup("vyys:");
  t_search3 = strdup("nopeus:");
  t_label2 = strdup("Wind Chill:");
  t_label3 = strdup("Wind Speed:");
  xlock_cmd = strdup("xlock &");
  keymap1_cmd = strdup("setxkeymap /var/X11/input/FI.xkm &");
  keymap2_cmd = strdup("setxkeymap /var/X11/input/US.xkm &");
  keymap1_name = strdup("fi");
  keymap2_name = strdup("us");
  keymap_label = strdup("Keyboard layout:");
  draw_border = 1;
  t_timeout = 5;
  clock1_dif=0;
  clock2_dif=-2;
  clock1_tz = strdup("EET");
  clock2_tz = strdup("UTC");
  RGB_mode = 0;
  pop3_enabled = 0;
  pop3_host = strdup("");
  pop3_user = strdup("");
}


void load_config(char *pname)
{
  FILE *fp;
  char buf[1024],*s, *port,*host,*loc;
  int i;
  

  set_defaults();

  fp=fopen(pname,"r");
  if (fp) {
    while(fgets(buf,sizeof(buf),fp)) {
      s=buf+strlen(buf)-1;
      while (s>=buf && (*s == 10 || *s==13)) *(s--)=0;
      s=buf;
      while (*s!=0 && (*s==' ' || *s=='\t')) s++;
      if (*s=='#' || *s==0) continue;
      
      i=0;
      while (confinfo[i++].tag) {
	char *r = strstr(s,confinfo[i-1].tag);
	if (!r) continue;
	r=r+strlen(confinfo[i-1].tag);
	if (*r++ == '=') {
	  // printf("'%s'\n",r);
	  if (confinfo[i-1].var) { /* string */
	    free(*(confinfo[i-1].var));
	    *(confinfo[i-1].var)=strdup(r);
	  } else if (confinfo[i-1].var_int) { /* int */
	    int tmp;
	    if (sscanf(r,"%d",&tmp)==1) {
	      *(confinfo[i-1].var_int)=tmp;
	    }
	  }
	}
      }
    }

    fclose(fp);
  }

  if (!split_url(t_url2,&host,&port,&loc)) {
    // printf("'%s' '%s' '%s'\n",host,port,loc);
    free(t_addr); t_addr=host;
    free(t_port); t_port=port;
    free(t_url); t_url=loc;
  }
  
  main_keymap1->label(keymap1_name);
  main_keymap2->label(keymap2_name);
  main_keymap_label->label(keymap_label);
  main_temp2->label(t_label2);
  main_temp3->label(t_label3);

  return;
}


int save_config(char *fname)
{
  FILE *fp;
  int i=0;

  fp=fopen(fname,"w");
  if (!fp) return -1;

  fprintf(fp,"# jyudesk configuration file\n#\n#\n");

  while (confinfo[i++].tag) {
    if (confinfo[i-1].var)
      fprintf(fp,"%s=%s\n",confinfo[i-1].tag,*(confinfo[i-1].var));
    else if (confinfo[i-1].var_int) 
      fprintf(fp,"%s=%d\n",confinfo[i-1].tag,*(confinfo[i-1].var_int));
  }
  fclose(fp);
  return 0;
}


//------------------------------------------------------------------
static int config_err = 0;

void config_cb(Fl_Button *, void *v)
{
  char *h,*p,*l;
  const char *val;
  int mode=(int)v;
  
  config_err=0;
  val=conf_temp_url->value();
  if (split_url(val,&h,&p,&l)) {
    fl_message("Invalid URL!!!");
    config_err=1;
    return;
  }
  free(t_url2); t_url2=strdup(val);
  free(t_url); t_url=l;
  free(t_port); t_port=p;
  free(t_addr); t_addr=h;

  free(t_search1); t_search1=strdup(conf_temp_search1->value());
  free(t_search2); t_search2=strdup(conf_temp_search2->value());
  free(t_search3); t_search3=strdup(conf_temp_search3->value());
  free(t_label2); t_label2=strdup(conf_temp_name2->value());
  free(t_label3); t_label3=strdup(conf_temp_name3->value());
  main_temp2->label(t_label2);
  main_temp3->label(t_label3);
  
  free(pb_addr); pb_addr=strdup(conf_pb_host->value());
  free(pb_port); pb_port=strdup(conf_pb_port->value());

  free(xlock_cmd); xlock_cmd=strdup(conf_xlock_cmd->value());

  free(keymap1_cmd); keymap1_cmd=strdup(conf_keymap1_cmd->value());
  free(keymap2_cmd); keymap2_cmd=strdup(conf_keymap2_cmd->value());

  free(keymap1_name); keymap1_name=strdup(conf_keymap1_name->value());
  free(keymap2_name); keymap2_name=strdup(conf_keymap2_name->value());
  free(keymap_label); keymap_label=strdup(conf_keymap_label->value());
  main_keymap1->label(keymap1_name);
  main_keymap2->label(keymap2_name);
  main_keymap_label->label(keymap_label);

  draw_border=(conf_border_button->value()?0:1);
  main_panel->border(draw_border);

  int m = (conf_RGB_button->value()?1:0);
  if (RGB_mode != m) 
    fl_message("Setting visual won't take effect before"
	       " you restart the program.");
  RGB_mode=m;

  int tmp;
  if (sscanf(conf_temp_timeout->value(),"%d",&tmp)==1) {
    if (tmp<1) tmp=1;
    t_timeout=tmp;
  }

  if (sscanf(conf_clock1_dif->value(),"%d",&tmp)==1)
    clock1_dif=tmp;
  if (sscanf(conf_clock2_dif->value(),"%d",&tmp)==1)
    clock2_dif=tmp;

  free(clock1_tz); clock1_tz=strdup(conf_clock1_tz->value());
  free(clock2_tz); clock2_tz=strdup(conf_clock2_tz->value());

  free(pop3_host); pop3_host=strdup(conf_pop_host->value());
  free(pop3_user); pop3_user=strdup(conf_pop_user->value());

  int tmpi=(conf_pop_r2->value()?1:0);
  if (tmpi && !pop3_pwd && !mode) pop3_pwd=ask_passwd();
  pop3_enabled=tmpi;
  if (!pop3_pwd && !mode) pop3_enabled=0;

  if (!mode) config_panel->hide();
  main_panel->redraw();
}


void config_show_cb(Fl_Object *, void*)
{
  char buf[256];

  conf_temp_url->value(t_url2);
  conf_temp_search1->value(t_search1);
  conf_temp_search2->value(t_search2);
  conf_temp_search3->value(t_search3);
  conf_temp_name2->value(t_label2);
  conf_temp_name3->value(t_label3);
  conf_pb_host->value(pb_addr);
  conf_pb_port->value(pb_port);
  conf_keymap1_cmd->value(keymap1_cmd);
  conf_keymap2_cmd->value(keymap2_cmd);
  conf_keymap1_name->value(keymap1_name);
  conf_keymap2_name->value(keymap2_name);
  conf_keymap_label->value(keymap_label);
  conf_xlock_cmd->value(xlock_cmd);
  conf_border_button->value(!draw_border);
  conf_RGB_button->value(RGB_mode);
  conf_clock1_tz->value(clock1_tz);
  conf_clock2_tz->value(clock2_tz);
  sprintf(buf,"%d",clock1_dif);
  conf_clock1_dif->value(buf);
  sprintf(buf,"%d",clock2_dif);
  conf_clock2_dif->value(buf);
  sprintf(buf,"%d",t_timeout);
  conf_temp_timeout->value(buf);
  conf_pop_host->value(pop3_host);
  conf_pop_user->value(pop3_user);
  conf_pop_r1->value(!pop3_enabled);
  conf_pop_r2->value(pop3_enabled);

  config_panel->show();
}


void config_save_cb(Fl_Button*, void*)
{
  config_cb(NULL,(void*)1);
  if (config_err) return;

  if (save_config(conf_file)) {
    fl_message("Cannot saved configuration!\nError writing to file:\n%s",
	       conf_file);
    return;
  }
  fl_message("Configuration save succesfully!\n%s",conf_file);
}
