/*********************************************************************
 * $Id: phonebook.C,v 1.11 1999/02/02 15:30:18 pjka Exp $
 *
 * Copyright (c) 1998  Timo Kokkonen <tjko@iki.fi>
 *
 *
 * This file may be copied under the terms and conditions 
 * of the GNU General Public License, as published by the Free
 * Software Foundation (Cambridge, Massachusetts).
 */

#include <stdio.h>
#include <string.h>

#include "main.H"

static char *rcsid = "$Id: phonebook.C,v 1.11 1999/02/02 15:30:18 pjka Exp $";

static S_Socket pb_socket, *pb_sock = &pb_socket;

char *pb_addr = NULL;
char *pb_port = NULL;

static char *query_str = NULL;

//---------------------------------------------------------------------

#define BUF_SIZE 8000

void phonebook_update(int fd, void *v)
{
  static char *data = NULL;
  static int data_size,data_res;
  int i = 0, wasread;
  int mode = (int)v;
  char *p1,*p2,*s;

  if (!pb_sock->connected()) {
    Fl::remove_fd(fd);
    return;
  }

  if (mode==1) {
    Fl::remove_fd(pb_sock->fd());
    int written = pb_sock->write(query_str,strlen(query_str));
    if (written < (int)strlen(query_str)) {
      fl_message("Cannot contact phonebook server:\n%s\n%s",
		 pb_sock->addresstr(),pb_sock->errstr());
      pb_sock->close();
      phone_browser->clear();
      show_info("");
      return;
    }
    Fl::add_fd(pb_sock->fd(),FL_READ,phonebook_update,0);
    // fprintf(stderr,"pb_sock->write() at %d\n",(int)time(NULL));
    return;
  }
  
  //------------------
  Fl::remove_fd(pb_sock->fd());

  if (!data) {
    data=(char*)malloc(BUF_SIZE+1);
    if (!data) exit(-2);
    data_size=BUF_SIZE; data_res=0;
  }

  do {
    if (data_res>=data_size) {
      data=(char*)realloc(data,data_size+BUF_SIZE);
      if (!data) exit(-2);
      data_size+=BUF_SIZE; 
    }
    wasread = pb_sock->read(data+data_res,data_size-data_res);
    if (wasread >= 0) {
      data_res+=wasread;
      if (pb_sock->error()==EAGAIN) {
	// fprintf(stderr,"foo %d\n",wasread);
	Fl::add_fd(pb_sock->fd(),FL_READ,phonebook_update,0);
	return;
      }
    } else {
      fl_message("Error reading data from socket!\n%s",
		 pb_sock->errstr());
    }

  } while (wasread > 0);


  phone_browser->clear();


  p1=data;
  p2=data+data_res;
  *(p2)=0;

  s=strtok(p1,"\r\n");
  if (!s) {
    phone_browser->add(""); 
    phone_browser->add("Nothing matching the search criteria found.");
  } else {
    while (s) {
      phone_browser->add(s);
      s=strtok(NULL,"\n");
      i++;
    }
  }

  free(data); data=NULL;
  pb_sock->close();
  phone_panel->show();
  show_info("");
}


void phonebook_cb(Fl_Return_Button *, void *)
{
  static int state = 0;
  static char buf[1024],query[80];
  static char *old_ip=NULL,*old_port=NULL;
  
  if (rcsid);
  if (!pb_sock->connected()) state=0;

  if (state==0) {
    phone_browser->clear();
    strncpy(query,phone_input->value(),sizeof(query));
    query[sizeof(query)-1]=0;
    if (strlen(query)<1) return;
    
    sprintf(buf,"%s\r\n",query);

    if (!old_ip || strcmp(old_ip,pb_addr)) {
      show_info("Hostname lookup...(%s)",pb_addr);
      Fl::wait();
      if (pb_sock->setaddress(pb_addr)<0) {
	fl_message("Invalid address!\n%s",pb_addr);
	show_info("");
	return;
      }
      show_info("");
      if (old_ip) free(old_ip); 
      old_ip=strdup(pb_addr);
    }

    if (!old_port || strcmp(old_port,pb_port)) {
      if (pb_sock->setport(pb_port)<0) {
	fl_message("Invalid port!\n%s",pb_port);
	return;
      }
      if (old_port) free(old_port);
      old_port=strdup(pb_port);
    }

    if (pb_sock->open(1) > 0) {
      phone_browser->add("");
      phone_browser->add("Please wait...");
      state=1;
      if (query_str) free(query_str);
      query_str=strdup(buf);

      Fl::add_fd(pb_sock->fd(),FL_WRITE,phonebook_update,(void*)1);
      // fprintf(stderr,"pb_sock->open() at %d\n",(int)time(NULL));
      show_info("Phonebook lookup...");
      return;
    } 
      
    fl_message("Cannot open socket!!!\n%s\n%s",pb_sock->errstr(),
	       pb_sock->addresstr());
    return;
  }
  else {
    fl_message("Previous query still being processed!");
  }

}


void show_phone_panel_cb(Fl_Button*, void*)
{
  phone_panel->show();
}
